#include <owl\owlpch.h>
#include <owl/applicat.h>
#include <classlib/time.h>
#include <owl/dialog.h>
#include <owl/framewin.h>
#include <owl/checkbox.h>
#include <owl/edit.h>
#include <owl/inputdia.h>
#include <classlib/listimp.h>
#include <cstring.h>
#include "logbook.rh"

//*** TDiaryEntry ***
class TDiaryEntry {
private:
  int starttime,endtime;
  TDate startdate;
  int mitternacht;
  string user;
  string bemerkung;
public:
  TDiaryEntry(char* s);
  void paint(TClientDC& dc,TDate enddate);
};

TDiaryEntry::TDiaryEntry(ifstream& in) {
  in.getline(user,' ');
  int day,month,year;
  in >> day >> month >> year >> mitternacht >> onhour >> onminute >> offhour >> offminute;
  in.getline(bemerkung,'\n');
  starttime=onhour*60+onminute;
  endtime=offhour*60+offminute;
  startdate(day,month,year)
};

//*** TDiaryArray ***
class TDiaryArray:public TArray<TDiaryEntry*> {
public:
  TDiaryArray(char* logbook);
  void paint(TClientDC& dc,TDate enddate);
};

TDiaryArray::TDiaryArray(char* logbook)
:TArray<TDiaryEntry*>(500,0,500) {
  ifstream in(logbook);
  while (!in.eof()) Add(new TDiaryEntry(in))
}

void TDiaryArray::Paint(TClientDC& dc,TDate enddate)
{

}

//*** TDiaryWindow ***
const TColor COL_BACK(255,255,255);
const TColor COL_LINE(128,128,128);
class TDiaryWindow: public TWindow {
private:
  TDiaryList List;
  TDate enddate;
public:
  TDiaryWindow (TWindow* parent,int x,int y,int w,int h,TDiaryList* list);
  virtual void Paint(TDC& dc, BOOL erase, TRect& rect);
};

TDiaryWindow::TDiaryWindow(TWindow* parent,int x,int y,int w,int h,TDiaryList* list)
:TWindow(parent,0,0) {
  List=list;
  new TGroupBox(parent,0,"",x-1, y-1,w+2,h+2);
  Attr.X=x;
  Attr.Y=y;
  Attr.W=w;
  Attr.H=h;
  SetBkgndColor(COL_BACK);
}

TDiaryWindow::Paint(TDC& adc, BOOL a, TRect& arec) {
  TWindow::Paint(adc,a,arec);
  TClientDC dc(*this);
  TBrush brush(COL_BACK);
  dc.SelectObject(brush);
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  dc.FillRect(rec,brush);
  TPen pen(COL_LINE);
  dc.SelectObject(pen);
  for (int x=rec.right-12; x>0; x+=12) {
	 dc.MoveTo(x,0);
	 dc.LineTo(x,rec.bottom);
  }
  int yh;
  for (int y=1; y<23; y++) {
	 yh=rec.bottom*y/24;
	 dc.MoveTo(0,yh);
	 dc.LineTo(rec.right,yh);
  }
  List->Paint(dc,enddate);
}

//*** TSchlussDialog ***
static const unsigned TextLen=48;
class TSchlussDialog: public TDialog {
public:
  class TBuffer {
	 public:
	 char Text[TextLen];
	 uint Check[5];
	 TBuffer ();
  };
  TSchlussDialog(TWindow*  parent,
					  TBuffer&	transferBuffer
					  );
};

TSchlussDialog::TBuffer::TBuffer() {
  Text[0]=0;
  for (int i=0;i<5;i++) Check[i]=BF_UNCHECKED;
}

TSchlussDialog::TSchlussDialog(TWindow*  parent,
										 TBuffer&  transferBuffer)
:
  TWindow(parent, "Bemerkung", 0),
  TDialog(parent,IDD_SCHLUSSDIALOG, 0)
{
  new TEdit(this, ID_BEMERKUNG, sizeof(transferBuffer.Text));
  new TCheckBox(this,IDC_MATHIAS);
  new TCheckBox(this,IDC_KATHARINA);
  new TCheckBox(this,IDC_FLORIAN);
  new TCheckBox(this,IDC_HANS);
  new TCheckBox(this,IDC_GAST);
  SetTransferBuffer(&transferBuffer);
}

//*** TLogBookDialog ***
const char[] logbookdat="c:/florian/logbook.dat";
class TLogbookWindow: public TFrameWindow {
private:
  TTime starttime;
  TDate startdate;
public:
  TLogbookWindow(const char* title);
  virtual bool CanClose();
  DECLARE_RESPONSE_TABLE(TLogbookWindow);
};

DEFINE_RESPONSE_TABLE1(TLogbookWindow, TFrameWindow)
END_RESPONSE_TABLE;

TLogbookWindow::TLogbookWindow(const char* title)
  :TFrameWindow(0,title),TWindow(0,title)
{
  new TDiaryWindow(this,10,10,640,500,logbookdat);
}

bool TLogbookWindow::CanClose()
{
  char Table[5]={'M','K','F','H','G'};
  TSchlussDialog::TBuffer Buffer;
  bool OK=false;
  while (!OK) {
	 if (TSchlussDialog(this,Buffer).Execute() != IDOK) return false;
	 for (int i=0;i<5;i++) OK|=Buffer.Check[i]==BF_CHECKED;
	 if (Buffer.Text[0]==0) OK=false;
  }
  TTime endtime;
  TDate enddate;
  ofstream book(logbookdat,ios::out|ios::app);
  for (int i=0;i<5;i++) {
	 if (Buffer.Check[i]==BF_CHECKED) book<<Table[i];
  }
  book << ' ' << startdate.DayOfMonth() << " " << startdate.Month() << " "
		 << startdate.Year() << " " << (enddate-startdate)
		 << " " << starttime.Hour() << " " << starttime.Minute() << " "
		 << " " << endtime.Hour() << " " << endtime.Minute() <<  " " << Buffer.Text << '\n';
  return true;
}

//*** LogbookApp ***
class TLogbookApp : public TApplication {
public:
  void InitMainWindow();
};

void TLogbookApp::InitMainWindow() {
  EnableCtl3d();
  EnableBWCC();
  if (nCmdShow != SW_HIDE)
		  nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWMINIMIZED : nCmdShow;
  TLogbookWindow* window=new TLogbookWindow("LogBook");
  window->SetIcon(this,IDI_LOGBOOK);
  window->Attr.W=200;
  window->Attr.H=0;
  SetMainWindow(window);
}

int OwlMain (int, char* []) {
  return TLogbookApp().Run();
}

